love.filesystem.setRequirePath ( "./?.dylib;" .. love.filesystem.getRequirePath() )

local lfs = require ( "lfs" )


local lg, li = love.graphics, love.image

local max, min = math.max, math.min

local images = {}
local imageNames = {}
local cropRects = {}
local frameNo = 0
local imgNo = 1

local HUGE = 1e10


local loadImage = function ( aSourcePath, anImageFileName )
	if anImageFileName:sub(-4) == ".png" then
		local f = io.open ( aSourcePath .. "/".. anImageFileName, "rb" )
		local data = f:read("*a")
		f:close()
		
		local f = io.open ( "temp.png", "wb" )
		f:write ( data )
		f:close()
		
		images [ #images + 1 ] = lg.newImage ( "temp.png" )
		imageNames [ #imageNames + 1 ] = anImageFileName
		
		os.remove ( "temp.png" )
	else
		local isDir = lfs.attributes ( aSourcePath .. "/" .. anImageFileName, "mode" ) == "directory" and
			anImageFileName ~= "." and anImageFileName ~= ".."
			
		print ( "Skipped '" .. anImageFileName .. "'", isDir )
	end
end


local loadImages = function ( aSourcePath )
	print ( "loading: ", aSourcePath )
	
	for imageFileName in lfs.dir ( aSourcePath ) do
		loadImage ( aSourcePath, imageFileName )
	end
end


local cropImage = function ( anImage )
	local iData = anImage:getData()
	
	local minX, minY = HUGE, HUGE
	local maxX, maxY = -1, -1
	
	local imgWd, imgHt = iData:getDimensions()
	
	for x = 0, imgWd-1 do
		for y = 0, imgHt-1 do
			local r, g, b, a = iData:getPixel ( x, y )
			if a ~= 0 then
				minX = min ( minX, x )
				minY = min ( minY, y )

				maxX = max ( maxX, x )
				maxY = max ( maxY, y )			
			end
		end
	end
	
	return { x = minX, y = minY, wd = maxX - minX, ht = maxY - minY }
end


local cropImages = function ( )
	print ( "Cropping:" )
	
	for imgNo, img in ipairs ( images ) do
		print ( "\t" .. imageNames [ imgNo ] )
		cropRects [ imgNo ] = cropImage ( img )
		cropRects [ imgNo ].name = imageNames [ imgNo ]
	end
end


local getCroppedImage = function ( anImgNo )
	local crop = cropRects [ anImgNo ]
	local iData = images [ anImgNo ]:getData ()
	
	local cImg = li.newImageData ( crop.wd, crop.ht )
	
	for x = 0, crop.wd-1 do
		for y = 0, crop.ht-1 do
			local r,g,b,a = iData:getPixel ( x + crop.x, y + crop.y )
			cImg:setPixel ( x, y, r, g, b, a )
		end
	end
	
	return cImg
end


local saveImages = function ( aTargetPath, aFolderName )
	for imgNo, img in ipairs ( images ) do
		local newImg = getCroppedImage ( imgNo )
		local crop = cropRects [ imgNo ]
		local cropImg = newImg:encode ( "png" )
		local f = io.open ( aTargetPath .. "/" .. crop.name, "wb" )
		f:write ( cropImg:getString () )
		f:close()
	end
	
	print ( string.format ( '\n\n\tlocal images = {}\n\timages["%s"] = {', aFolderName ) )
	for _, r in ipairs ( cropRects ) do
		print ( string.format ( '\t\t{ %-20s = { x = %4d, y = %4d, wd = %4d, ht = %4d } },', r.name:sub(1,-5), 
			r.x, r.y, r.wd, r.ht ) )
	end
	print ( "\t}")
end


processDirectory = function ( aPath, aFolderName )
	images = {}
	imageNames = {}
	cropRects = {}
		
	loadImages ( aPath )
	cropImages ()
		
	local targetPath = aPath .. "/Cropped"
	lfs.mkdir ( targetPath )
	saveImages ( targetPath, aFolderName )
end


love.load = function ( )
	love.window.setMode ( 1920, 1080 )
end


love.filedropped = function ( aFile )
	print ("File ", aFile:getFilename() )
	
	local path, fName = aFile:getFilename():match("(.-)([^\\/]-%.?([^%.\\/]*))$")
	path = path:sub(1, -2)
	
	local subPath, folder = path:match("(.-)([^\\/]-%.?([^%.\\/]*))$")
	print ( folder )
	
	images = {}
	imageNames = {}
	cropRects = {}
		
	loadImage ( path, fName )
	cropImages ()
	
	local targetPath = path .. "/Cropped"
	lfs.mkdir ( targetPath )
	saveImages ( targetPath, folder )
end


love.directorydropped = function ( aPath )
	local subPath, folder = aPath:match("(.-)([^\\/]-%.?([^%.\\/]*))$")
	print ( folder )
	
	processDirectory ( aPath, folder )
end


love.update = function ( dt )
	frameNo = frameNo + 1
	if frameNo % 3 == 0 then 
		imgNo = imgNo + 1
		if imgNo > #images then imgNo = 1 end
	end
end


love.draw = function ( )
	if images [ imgNo ] then
		lg.setColor ( 255, 255, 255, 255 )
		lg.draw ( images [ imgNo ], 0, 0 )
		
		local r = cropRects [ imgNo ]
		lg.setColor ( 0, 255, 255, 255 )
		lg.rectangle ( "line", r.x, r.y, r.wd, r.ht )
	end
end

love.keypressed = function ( aKey ) if aKey == "escape" then love.event.quit() end end


print ("Done.")
	